@@ -1,12 +1,13 @@
 # coding=utf-8
 #
-# This file is part of Hypothesis (https://github.com/DRMacIver/hypothesis)
+# This file is part of Hypothesis, which may be found at
+# https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
-# https://github.com/DRMacIver/hypothesis/blob/master/CONTRIBUTING.rst for a
-# full list of people who may hold copyright, and consult the git log if you
-# need to determine who owns an individual contribution.
+# CONTRIBUTING.rst for a full list of people who may hold copyright, and
+# consult the git log if you need to determine who owns an individual
+# contribution.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
@@ -19,8 +20,10 @@ from __future__ import division, print_function, absolute_import
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import NoExamples, NoSuchExample, Unsatisfiable, \
     UnsatisfiedAssumption
-from hypothesis.control import assume, reject
+from hypothesis.control import assume, reject, _current_build_context
+from hypothesis._settings import note_deprecation
 from hypothesis.internal.compat import hrange
+from hypothesis.internal.lazyformat import lazyformat
 from hypothesis.internal.reflection import get_pretty_function_description
 
 
@@ -29,8 +32,7 @@ def one_of_strategies(xs):
     xs = tuple(xs)
     if not xs:
         raise ValueError('Cannot join an empty list of strategies')
-    from hypothesis.strategies import one_of
-    return one_of(xs)
+    return OneOfStrategy(xs)
 
 
 class SearchStrategy(object):
@@ -72,8 +74,6 @@ class SearchStrategy(object):
     hashable probably won't be.
 
     This may all seem overly complicated but it's for a fairly good reason.
-    For more discussion of the motivation see
-    http://hypothesis.readthedocs.org/en/master/internals.html
 
     Given these, data generation happens in three phases:
 
@@ -89,7 +89,45 @@ class SearchStrategy(object):
     """
 
     supports_find = True
-    is_empty = False
+    validate_called = False
+
+    def recursive_property(name, default):
+        cache_key = 'cached_' + name
+        calculation = 'calc_' + name
+        force_key = 'force_' + name
+
+        def accept(self):
+            if not hasattr(self, cache_key):
+                try:
+                    setattr(self, cache_key, getattr(self, force_key))
+                except AttributeError:
+                    # This isn't an error, but instead is to deal with
+                    # recursive strategy definitions that refer to themselves.
+                    # This ensures in a recursive call we will return the
+                    # default value.
+                    setattr(self, cache_key, default)
+                    setattr(self, cache_key, getattr(self, calculation)())
+            return getattr(self, cache_key)
+
+        accept.__name__ = name
+        return property(accept)
+
+    # Returns True if this strategy can never draw a value and will always
+    # result in the data being marked invalid.
+    # The fact that this returns False does not guarantee that a valid value
+    # can be drawn - this is not intended to be perfect, and is primarily
+    # intended to be an optimisation for some cases.
+    is_empty = recursive_property('is_empty', False)
+
+    # Returns True if values from this strategy can safely be reused without
+    # this causing unexpected behaviour.
+    has_reusable_values = recursive_property('has_reusable_values', True)
+
+    def calc_is_empty(self):
+        return False
+
+    def calc_has_reusable_values(self):
+        return False
 
     def example(self, random=None):
         """Provide an example of the sort of value that this strategy
@@ -102,7 +140,35 @@ class SearchStrategy(object):
         This method is part of the public API.
 
         """
-        from hypothesis import find, settings
+        context = _current_build_context.value
+        if context is not None:
+            if context.data is not None and context.data.depth > 0:
+                note_deprecation(
+                    'Using example() inside a strategy definition is a bad '
+                    'idea. It will become an error in a future version of '
+                    "Hypothesis, but it's unlikely that it's doing what you "
+                    'intend even now. Instead consider using '
+                    'hypothesis.strategies.builds() or '
+                    '@hypothesis.strategies.composite to define your strategy.'
+                    ' See '
+                    'https://hypothesis.readthedocs.io/en/latest/data.html'
+                    '#hypothesis.strategies.builds or '
+                    'https://hypothesis.readthedocs.io/en/latest/data.html'
+                    '#composite-strategies for more details.'
+                )
+            else:
+                note_deprecation(
+                    'Using example() inside a test function is a bad '
+                    'idea. It will become an error in a future version of '
+                    "Hypothesis, but it's unlikely that it's doing what you "
+                    'intend even now. Instead consider using '
+                    'hypothesis.strategies.data() to draw '
+                    'more examples during testing. See '
+                    'https://hypothesis.readthedocs.io/en/latest/data.html'
+                    '#drawing-interactively-in-tests for more details.'
+                )
+
+        from hypothesis import find, settings, Verbosity
         try:
             return find(
                 self,
@@ -111,7 +177,8 @@ class SearchStrategy(object):
                 settings=settings(
                     max_shrinks=0,
                     max_iterations=1000,
-                    database=None
+                    database=None,
+                    verbosity=Verbosity.quiet,
                 )
             )
         except (NoSuchExample, Unsatisfiable):
@@ -157,6 +224,10 @@ class SearchStrategy(object):
             strategy=self,
         )
 
+    @property
+    def branches(self):
+        return [self]
+
     def __or__(self, other):
         """Return a strategy which produces values by randomly drawing from one
         of this strategy or the other strategy.
@@ -166,16 +237,24 @@ class SearchStrategy(object):
         """
         if not isinstance(other, SearchStrategy):
             raise ValueError('Cannot | a SearchStrategy with %r' % (other,))
-        if other.is_empty:
-            return self
         return one_of_strategies((self, other))
 
     def validate(self):
-        """Through an exception if the strategy is not valid.
+        """Throw an exception if the strategy is not valid.
 
         This can happen due to lazy construction
 
         """
+        if self.validate_called:
+            return
+        try:
+            self.validate_called = True
+            self.do_validate()
+        except:
+            self.validate_called = False
+            raise
+
+    def do_validate(self):
         pass
 
     def do_draw(self, data):
@@ -190,43 +269,85 @@ class OneOfStrategy(SearchStrategy):
     """Implements a union of strategies. Given a number of strategies this
     generates values which could have come from any of them.
 
-    The conditional distribution draws uniformly at random from some non-empty
-    subset of these strategies and then draws from the conditional distribution
-    of that strategy.
+    The conditional distribution draws uniformly at random from some
+    non-empty subset of these strategies and then draws from the
+    conditional distribution of that strategy.
 
     """
 
     def __init__(self, strategies, bias=None):
         SearchStrategy.__init__(self)
         strategies = tuple(strategies)
-        self.element_strategies = list(strategies)
+        self.original_strategies = list(strategies)
+        self.__element_strategies = None
         self.bias = bias
+        self.__in_branches = False
         if bias is not None:
             assert 0 < bias < 1
-            self.weights = [bias ** i for i in range(len(strategies))]
+            self.sampler = cu.Sampler(
+                [bias ** i for i in range(len(strategies))])
+        else:
+            self.sampler = None
+
+    def calc_is_empty(self):
+        return len(self.element_strategies) == 0
+
+    def calc_has_reusable_values(self):
+        return all(e.has_reusable_values for e in self.element_strategies)
+
+    @property
+    def element_strategies(self):
+        from hypothesis.strategies import check_strategy
+        if self.__element_strategies is None:
+            strategies = []
+            for arg in self.original_strategies:
+                check_strategy(arg)
+                if not arg.is_empty:
+                    strategies.extend(
+                        [s for s in arg.branches if not s.is_empty])
+            pruned = []
+            seen = set()
+            for s in strategies:
+                if s is self:
+                    continue
+                if s in seen:
+                    continue
+                seen.add(s)
+                pruned.append(s)
+            self.__element_strategies = pruned
+        return self.__element_strategies
 
     def do_draw(self, data):
         n = len(self.element_strategies)
-        if self.bias is None:
+        if n == 0:
+            data.mark_invalid()
+        elif n == 1:
+            return data.draw(self.element_strategies[0])
+        elif self.sampler is None:
             i = cu.integer_range(data, 0, n - 1)
         else:
-            def biased_i(random):
-                while True:
-                    i = random.randint(0, n - 1)
-                    if random.random() <= self.weights[i]:
-                        return i
-            i = cu.integer_range_with_distribution(
-                data, 0, n - 1, biased_i)
+            i = self.sampler.sample(data)
 
         return data.draw(self.element_strategies[i])
 
     def __repr__(self):
-        return ' | '.join(map(repr, self.element_strategies))
+        return ' | '.join(map(repr, self.original_strategies))
 
-    def validate(self):
+    def do_validate(self):
         for e in self.element_strategies:
             e.validate()
 
+    @property
+    def branches(self):
+        if self.bias is None and not self.__in_branches:
+            try:
+                self.__in_branches = True
+                return self.element_strategies
+            finally:
+                self.__in_branches = False
+        else:
+            return [self]
+
 
 class MappedSearchStrategy(SearchStrategy):
 
@@ -242,7 +363,9 @@ class MappedSearchStrategy(SearchStrategy):
         self.mapped_strategy = strategy
         if pack is not None:
             self.pack = pack
-        self.is_empty = strategy.is_empty
+
+    def calc_is_empty(self):
+        return self.mapped_strategy.is_empty
 
     def __repr__(self):
         if not hasattr(self, '_cached_repr'):
@@ -252,7 +375,7 @@ class MappedSearchStrategy(SearchStrategy):
             )
         return self._cached_repr
 
-    def validate(self):
+    def do_validate(self):
         self.mapped_strategy.validate()
 
     def pack(self, x):
@@ -265,12 +388,19 @@ class MappedSearchStrategy(SearchStrategy):
         for _ in range(3):
             i = data.index
             try:
-                return self.pack(self.mapped_strategy.do_draw(data))
+                return self.pack(data.draw(self.mapped_strategy))
             except UnsatisfiedAssumption:
                 if data.index == i:
                     raise
         reject()
 
+    @property
+    def branches(self):
+        return [
+            MappedSearchStrategy(pack=self.pack, strategy=strategy)
+            for strategy in self.mapped_strategy.branches
+        ]
+
 
 class FilteredStrategy(SearchStrategy):
 
@@ -278,7 +408,9 @@ class FilteredStrategy(SearchStrategy):
         super(FilteredStrategy, self).__init__()
         self.condition = condition
         self.filtered_strategy = strategy
-        self.is_empty = strategy.is_empty
+
+    def calc_is_empty(self):
+        return self.filtered_strategy.is_empty
 
     def __repr__(self):
         if not hasattr(self, '_cached_repr'):
@@ -288,18 +420,32 @@ class FilteredStrategy(SearchStrategy):
             )
         return self._cached_repr
 
-    def validate(self):
+    def do_validate(self):
         self.filtered_strategy.validate()
 
     def do_draw(self, data):
-        for _ in hrange(3):
+        for i in hrange(3):
             start_index = data.index
             value = data.draw(self.filtered_strategy)
             if self.condition(value):
                 return value
             else:
+                if i == 0:
+                    data.note_event(lazyformat(
+                        'Retried draw from %r to satisfy filter', self,))
                 # This is to guard against the case where we consume no data.
                 # As long as we consume data, we'll eventually pass or raise.
                 # But if we don't this could be an infinite loop.
                 assume(data.index > start_index)
+        data.note_event('Aborted test because unable to satisfy %r' % (
+            self,
+        ))
         data.mark_invalid()
+
+    @property
+    def branches(self):
+        branches = [
+            FilteredStrategy(strategy=strategy, condition=self.condition)
+            for strategy in self.filtered_strategy.branches
+        ]
+        return branches
